/*
 * webalizer_lang_shared.c
 *
 * Copyright (C) 2000 Hiroki Ata<hata@kondara.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include <malloc.h>

#include "webalizer_lang_shared.h"


/**** LANG_SUFFIX is applied by Makefile or scripts for each Languages. ****/
#ifndef LANG_SUFFIX
# include "webalizer_lang._REPLACED_LANG_SUFFIX_"
#else
# include "webalizer_lang." LANG_SUFFIX
#endif /* LANG_SUFFIX */


#define TBL_KEY   char*
#define TBL_DATA  char*
#define TBL_SIZE  100


#define ADD_MSG_TABLE(key, value) _msg_tbl_insert(key, value)
#define WAR_MSG(msg)        //  fprintf(stderr, "Warrning:" msg "\n");
#define WAR_MSG_1(msg,s1)   //  fprintf(stderr, "Warrning:" msg "\n", s1);

#ifndef MAX_CTRY
#define MAX_CTRY (int)(sizeof(ctry)/sizeof(struct country_code))
#endif /* MAX_CTRY */

#ifndef LAST_HLP_MSG
#define LAST_HLP_MSG (int)(sizeof(h_msg)/sizeof(char *))
#endif /* LAST_HLP_MSG */

/* a cell for hash table. */
typedef struct tbl_cell {
	TBL_KEY          key;
	TBL_DATA         data;
	struct tbl_cell *next;
} Cell;


/* key and messages. This values are managed by hashtable. */
static Cell *msg_table[TBL_SIZE];


/* the table "msg_table" msg_tbl_initialized flag. */
static int   msg_tbl_initialized = 0;


/* initialize hash table. must call before using msg table. */
void _msg_tbl_init() {
	int i;
	for (i = 0;i < TBL_SIZE;i++)
		msg_table[i] = NULL;
}


/* key string to hash value. mu... */
int _msg_tbl_hash (const TBL_KEY key) {
	int i = 0;

	while (*key)
		i += *key++;
	
	return i % TBL_SIZE;
}


/* compare keys. */
int _msg_tbl_keyequal (const TBL_KEY key1,const TBL_KEY key2) {
	return !strcmp (key1, key2);
}


/* find message text from key. */
const char* _msg_tbl_find(const TBL_KEY key) {
	Cell *p;

	for (p = msg_table[_msg_tbl_hash(key)];p != NULL;p = p->next) {
		if (_msg_tbl_keyequal (key, p->key)){
			return p->data;
		}
	}

	WAR_MSG_1 ("finding message not found. message key is =%s", key);
	return NULL;
}


/* insert new key entry. */
void _msg_tbl_insert (TBL_KEY key, TBL_DATA data) {
	Cell* p;
	int h;

	if (_msg_tbl_find (key) != NULL) {
		WAR_MSG_1 ("Key=%s is already inserted.", key);
		return;
	} else if ((p = (Cell*)malloc (sizeof(Cell))) == NULL) {
		WAR_MSG ("Can not alloc memory.\n");
		return;
	}

	h = _msg_tbl_hash (key);
	
	p->key = malloc (strlen(key) + 1);
	strncpy (p->key, key, strlen(key));
	p->key[strlen(key)] = '\0';

	p->data = malloc (strlen(data) + 1);
	strncpy (p->data, data, strlen(data));
	p->data[strlen(data)] = '\0';

	p->next = msg_table[h];
	msg_table[h] = p;
}


/* delete all entries. */
void _msg_tbl_delete_all () {
	int i;
	Cell* p;
	Cell* next;

	for (i = 0;i < TBL_SIZE;i++) {
		p = msg_table[i];
		while (p) {
			next = p->next;
			free (p->key);
			free (p->data);
			free (p);
			p = next;
		}
		msg_table[i] = NULL;
	}
}


/* a client must call for initialization.
   If initialization is failed, this function return 0. */
int init_lang_module ()
{
	if (!msg_tbl_initialized) {
		_msg_tbl_init();

		ADD_MSG_TABLE("language",language);

		ADD_MSG_TABLE("msg_records",msg_records);
		ADD_MSG_TABLE("msg_addresses",msg_addresses);
		ADD_MSG_TABLE("msg_ignored",msg_ignored);
		ADD_MSG_TABLE("msg_bad",msg_bad);
		ADD_MSG_TABLE("msg_in",msg_in);
		ADD_MSG_TABLE("msg_seconds",msg_seconds);

		ADD_MSG_TABLE("msg_log_err",msg_log_err);
		ADD_MSG_TABLE("msg_log_use",msg_log_use);
		ADD_MSG_TABLE("msg_dir_err",msg_dir_err);
		ADD_MSG_TABLE("msg_dir_use",msg_dir_use);
		ADD_MSG_TABLE("msg_cur_dir",msg_cur_dir);
		ADD_MSG_TABLE("msg_hostname",msg_hostname);
		ADD_MSG_TABLE("msg_ign_hist",msg_ign_hist);
		ADD_MSG_TABLE("msg_no_hist",msg_no_hist);
		ADD_MSG_TABLE("msg_get_hist",msg_get_hist);
		ADD_MSG_TABLE("msg_put_hist",msg_put_hist);
		ADD_MSG_TABLE("msg_hist_err",msg_hist_err);
		ADD_MSG_TABLE("msg_bad_hist",msg_bad_hist);
		ADD_MSG_TABLE("msg_bad_conf",msg_bad_conf);
		ADD_MSG_TABLE("msg_bad_key",msg_bad_key);
		ADD_MSG_TABLE("msg_bad_date",msg_bad_date);
		ADD_MSG_TABLE("msg_ign_nscp",msg_ign_nscp);
		ADD_MSG_TABLE("msg_bad_rec",msg_bad_rec);
		ADD_MSG_TABLE("msg_no_vrec",msg_no_vrec);
		ADD_MSG_TABLE("msg_gen_rpt",msg_gen_rpt);
		ADD_MSG_TABLE("msg_gen_sum",msg_gen_sum);
		ADD_MSG_TABLE("msg_get_data",msg_get_data);
		ADD_MSG_TABLE("msg_put_data",msg_put_data);
		ADD_MSG_TABLE("msg_no_data",msg_no_data);
		ADD_MSG_TABLE("msg_bad_data",msg_bad_data);
		ADD_MSG_TABLE("msg_data_err",msg_data_err);
		ADD_MSG_TABLE("msg_dup_data",msg_dup_data);

		ADD_MSG_TABLE("msg_dns_nocf",msg_dns_nocf);
		ADD_MSG_TABLE("msg_dns_nodb",msg_dns_nodb);
		ADD_MSG_TABLE("msg_dns_nolk",msg_dns_nolk);
		ADD_MSG_TABLE("msg_dns_usec",msg_dns_usec);
		ADD_MSG_TABLE("msg_dns_rslv",msg_dns_rslv);
		ADD_MSG_TABLE("msg_dns_none",msg_dns_none);

		ADD_MSG_TABLE("msg_nomem_ts",msg_nomem_ts);
		ADD_MSG_TABLE("msg_nomem_tr",msg_nomem_tr);
		ADD_MSG_TABLE("msg_nomem_tu",msg_nomem_tu);
		ADD_MSG_TABLE("msg_nomem_tc",msg_nomem_tc);
		ADD_MSG_TABLE("msg_nomem_ta",msg_nomem_ta);
		ADD_MSG_TABLE("msg_nomem_tsr",msg_nomem_tsr);
		ADD_MSG_TABLE("msg_nomem_ti",msg_nomem_ti);
		ADD_MSG_TABLE("msg_nomem_dh",msg_nomem_dh);
		ADD_MSG_TABLE("msg_nomem_mh",msg_nomem_mh);
		ADD_MSG_TABLE("msg_nomem_u",msg_nomem_u);
		ADD_MSG_TABLE("msg_nomem_a",msg_nomem_a);
		ADD_MSG_TABLE("msg_nomem_r",msg_nomem_r);
		ADD_MSG_TABLE("msg_nomem_sc",msg_nomem_sc);
		ADD_MSG_TABLE("msg_nomem_i",msg_nomem_i);

		ADD_MSG_TABLE("msg_big_rec",msg_big_rec);
		ADD_MSG_TABLE("msg_big_host",msg_big_host);
		ADD_MSG_TABLE("msg_big_date",msg_big_date);
		ADD_MSG_TABLE("msg_big_req",msg_big_req);
		ADD_MSG_TABLE("msg_big_ref",msg_big_ref);
		ADD_MSG_TABLE("msg_big_user",msg_big_user);
		ADD_MSG_TABLE("msg_big_one",msg_big_one);

		ADD_MSG_TABLE("msg_no_open",msg_no_open);
		ADD_MSG_TABLE("h_usage1",h_usage1);
		ADD_MSG_TABLE("h_usage2",h_usage2);

		ADD_MSG_TABLE("msg_hhdr_sp",msg_hhdr_sp);
		ADD_MSG_TABLE("msg_hhdr_gt",msg_hhdr_gt);

		ADD_MSG_TABLE("msg_main_us",msg_main_us);
		ADD_MSG_TABLE("msg_main_per",msg_main_per);
		ADD_MSG_TABLE("msg_main_sum",msg_main_sum);
		ADD_MSG_TABLE("msg_main_da",msg_main_da);
		ADD_MSG_TABLE("msg_main_mt",msg_main_mt);

		ADD_MSG_TABLE("msg_hmth_du",msg_hmth_du);
		ADD_MSG_TABLE("msg_hmth_hu",msg_hmth_hu);

		ADD_MSG_TABLE("msg_h_by",msg_h_by);
		ADD_MSG_TABLE("msg_h_max",msg_h_max);
		ADD_MSG_TABLE("msg_h_avg",msg_h_avg);
		ADD_MSG_TABLE("msg_h_total",msg_h_total);
		ADD_MSG_TABLE("msg_h_totals",msg_h_totals);
		ADD_MSG_TABLE("msg_h_day",msg_h_day);
		ADD_MSG_TABLE("msg_h_mth",msg_h_mth);
		ADD_MSG_TABLE("msg_h_hour",msg_h_hour);
		ADD_MSG_TABLE("msg_h_hits",msg_h_hits);
		ADD_MSG_TABLE("msg_h_pages",msg_h_pages);
		ADD_MSG_TABLE("msg_h_visits",msg_h_visits);
		ADD_MSG_TABLE("msg_h_files",msg_h_files);
		ADD_MSG_TABLE("msg_h_sites",msg_h_sites);
		ADD_MSG_TABLE("msg_h_xfer",msg_h_xfer);
		ADD_MSG_TABLE("msg_h_hname",msg_h_hname);
		ADD_MSG_TABLE("msg_h_url",msg_h_url);
		ADD_MSG_TABLE("msg_h_agent",msg_h_agent);
		ADD_MSG_TABLE("msg_h_ref",msg_h_ref);
		ADD_MSG_TABLE("msg_h_ctry",msg_h_ctry);
		ADD_MSG_TABLE("msg_h_search",msg_h_search);
		ADD_MSG_TABLE("msg_h_uname",msg_h_uname);

		ADD_MSG_TABLE("msg_hlnk_ds",msg_hlnk_ds);
		ADD_MSG_TABLE("msg_hlnk_hs",msg_hlnk_hs);
		ADD_MSG_TABLE("msg_hlnk_u",msg_hlnk_u);
		ADD_MSG_TABLE("msg_hlnk_s",msg_hlnk_s);
		ADD_MSG_TABLE("msg_hlnk_a",msg_hlnk_a);
		ADD_MSG_TABLE("msg_hlnk_c",msg_hlnk_c);
		ADD_MSG_TABLE("msg_hlnk_r",msg_hlnk_r);
		ADD_MSG_TABLE("msg_hlnk_en",msg_hlnk_en);
		ADD_MSG_TABLE("msg_hlnk_ex",msg_hlnk_ex);
		ADD_MSG_TABLE("msg_hlnk_sr",msg_hlnk_sr);
		ADD_MSG_TABLE("msg_hlnk_i",msg_hlnk_i);

		ADD_MSG_TABLE("msg_mtot_ms",msg_mtot_ms);
		ADD_MSG_TABLE("msg_mtot_th",msg_mtot_th);
		ADD_MSG_TABLE("msg_mtot_tf",msg_mtot_tf);
		ADD_MSG_TABLE("msg_mtot_tx",msg_mtot_tx);
		ADD_MSG_TABLE("msg_mtot_us",msg_mtot_us);
		ADD_MSG_TABLE("msg_mtot_ur",msg_mtot_ur);
		ADD_MSG_TABLE("msg_mtot_ua",msg_mtot_ua);
		ADD_MSG_TABLE("msg_mtot_uu",msg_mtot_uu);
		ADD_MSG_TABLE("msg_mtot_ui",msg_mtot_ui);
		ADD_MSG_TABLE("msg_mtot_mhd",msg_mtot_mhd);
		ADD_MSG_TABLE("msg_mtot_mhh",msg_mtot_mhh);
		ADD_MSG_TABLE("msg_mtot_mfd",msg_mtot_mfd);
		ADD_MSG_TABLE("msg_mtot_mpd",msg_mtot_mpd);
		ADD_MSG_TABLE("msg_mtot_mvd",msg_mtot_mvd);
		ADD_MSG_TABLE("msg_mtot_mkd",msg_mtot_mkd);
		ADD_MSG_TABLE("msg_mtot_rc",msg_mtot_rc);


		ADD_MSG_TABLE("msg_dtot_ds",msg_dtot_ds);

		ADD_MSG_TABLE("msg_htot_hs",msg_htot_hs);


		ADD_MSG_TABLE("msg_ctry_use",msg_ctry_use);

		ADD_MSG_TABLE("msg_top_top",msg_top_top);
		ADD_MSG_TABLE("msg_top_of",msg_top_of);
		ADD_MSG_TABLE("msg_top_s",msg_top_s);
		ADD_MSG_TABLE("msg_top_u",msg_top_u);
		ADD_MSG_TABLE("msg_top_r",msg_top_r);
		ADD_MSG_TABLE("msg_top_a",msg_top_a);
		ADD_MSG_TABLE("msg_top_c",msg_top_c);
		ADD_MSG_TABLE("msg_top_en",msg_top_en);
		ADD_MSG_TABLE("msg_top_ex",msg_top_ex);
		ADD_MSG_TABLE("msg_top_sr",msg_top_sr);
		ADD_MSG_TABLE("msg_top_i",msg_top_i);
		ADD_MSG_TABLE("msg_v_sites",msg_v_sites);
		ADD_MSG_TABLE("msg_v_urls",msg_v_urls);
		ADD_MSG_TABLE("msg_v_refs",msg_v_refs);
		ADD_MSG_TABLE("msg_v_agents",msg_v_agents);
		ADD_MSG_TABLE("msg_v_search",msg_v_search);
		ADD_MSG_TABLE("msg_v_users",msg_v_users);

		ADD_MSG_TABLE("msg_title",msg_title);
		ADD_MSG_TABLE("msg_h_other",msg_h_other);

		
/* special messages for webalizer_lang.japanese. */
#ifdef japanese
			ADD_MSG_TABLE("msg_top_all",msg_top_all);
			ADD_MSG_TABLE("msg_hhdr_date",msg_hhdr_date);
			ADD_MSG_TABLE("msg_hhdr_gmtdate",msg_hhdr_gmtdate);
			ADD_MSG_TABLE("msg_year",msg_year);
#endif /* japanese */

		msg_tbl_initialized = 1;
		return 1;
	}else {
		return 0;
	}
}


/* a client must call for free datas. */
void fini_lang_module ()
{
	if (msg_tbl_initialized) {
		_msg_tbl_delete_all ();
		msg_tbl_initialized = 0;
	}else{
	}
}


/* get message text from key. */
const char* get_msg(const char* pKey)
{
	return _msg_tbl_find(pKey);
}


/* get country code from idx value. */

const char* get_country_code_desc (u_long idx)
{
	int i = 0;
	while (ctry[i].desc) {
		if (ctry[i].idx == idx)
			return ctry[i].desc;
		else
			i++;
	}
	return NULL;
}


/* get response code description from index. */
const char* get_response_code_desc (int n)
{
	return 0 <= n && n < (int)sizeof(response)/sizeof(struct response_code)
		? response[n].desc : NULL;
}


/* get month desc table. from index (0 - 11). */
char** get_short_month_tbl()
{
	return s_month;
}


/* get month desc table from index (0 - 11). */
char** get_long_month_tbl()
{
	return l_month;
}


/* get help message. */
const char* get_help_message(int n)
{
	return 0 <= n && n < ((int)(sizeof(h_msg)/sizeof(char *)))
		? *(h_msg + n) : NULL;
}

/* get help msg max index + 1. */
int         get_last_hlp_msg()
{
	return LAST_HLP_MSG;
}


/* get country code max index + 1. */
int         get_max_ctry()
{
	return MAX_CTRY;
}

